/* $Id: getaddrinfo-fake.c,v 1.2 2002/03/01 02:26:14 d Exp $ */

/*
 * faked getaddrinfo
 * - only works for IPv4 dotted quads and numeric TCP ports
 * - beggars can't be choosers. if your system has a real getaddrinfo()
 *   then you get full DNS, IPv6 and so forth. I'm not going to write
 *   all that.
 *
 * Based on interface described in:
 *   W R Stevens, Unix Network Programming: Networking APIs: Sockets and XTI,
 *   2nd ed., Volume 1, Prentice-Hall,Upper Daddle River, New Jersey 1998.
 */

#include <netinet/in.h>

#define addrinfo        Faddrinfo
#define gai_strerror    Fgai_strerror
#define freeaddrinfo    Ffreeaddrinfo
#define getaddrinfo     Fgetaddrinfo
#define getnameinfo     Fgetnameinfo
#undef NI_MAXHOST
#undef NI_MAXSERV
#undef AI_PASSIVE


#define NI_MAXHOST 12
#define NI_MAXSERV 6

struct addrinfo {
    int     ai_flags;     /* AI_PASSIVE, AI_CANONNAME, AI_NUMERICHOST */
#define AI_PASSIVE 9
    int     ai_family;    /* PF_xxx */
    int     ai_socktype;  /* SOCK_xxx */
    int     ai_protocol;  /* 0 or IPPROTO_xxx for IPv4 and IPv6 */
    size_t  ai_addrlen;   /* length of ai_addr */
    char   *ai_canonname; /* canonical name for nodename */
    struct sockaddr  *ai_addr; /* binary address */
    struct addrinfo  *ai_next; /* next structure in linked list */
};

static char *
gai_strerror(int error)
{
	return "unknown error";
}

static void
freeaddrinfo(struct addrinfo *ai)
{
}

static int
getaddrinfo(const char *nodename, const char *servname,
    const struct addrinfo *hints, struct addrinfo **res)
{
	static struct addrinfo a;
	static struct sockaddr_in addr;

	*res = &a;
	a.ai_flags = 0;
	a.ai_family = PF_INET;
	a.ai_socktype = SOCK_STREAM;
	a.ai_protocol = 0;
	a.ai_addrlen = sizeof addr;
	a.ai_canonname = NULL;
	a.ai_addr = (struct sockaddr *)&addr;
	a.ai_next = NULL;
	if (nodename == NULL)
		addr.sin_addr.s_addr = INADDR_ANY;
	else if (!inet_aton(nodename, &addr.sin_addr))
		return 1;
	addr.sin_port = htons(atoi(servname));
	return 0;
}

static int
getnameinfo(const struct sockaddr *sa, socklen_t salen, char *host,
    size_t hostlen, char *serv, size_t servlen, int flags)
{
	struct sockaddr_in *addr = (struct sockaddr_in *)sa;

	snprintf(serv, servlen, "%u", ntohs(addr->sin_port));
	snprintf(host, hostlen, "%s", inet_ntoa(addr->sin_addr));
	return 0;
}
